/* -------------------------------------------- */
import { HtmlUtility } from "./html-utility.js";
import { RdDUtility } from "./rdd-utility.js";

/* -------------------------------------------- */
export class RdDTokenHud {


  /* -------------------------------------------- */
  static async addTokenHudExtensions(app, html, tokenId) {
    let token = canvas.tokens.get(tokenId);
    let actor = token.actor;
    let combatant = game.combat.data.combatants.find(c => c.tokenId == token.data._id);

    // Affichage seulement si le token est un combat et a un actor
    if (actor === undefined) return;
    if (!token.inCombat) return;
    if (!combatant) return;

    let armesList = RdDTokenHud.buildListeActionsCombat(combatant) ;
    const hudData = { combatant: combatant, armes: armesList };

    // initiative
    await RdDTokenHud._configureSubMenu(html.find('.control-icon.combat'), 'systems/foundryvtt-reve-de-dragon/templates/hud-actor-init.html', hudData,
      (event) => {
        let armeIndex = event.currentTarget.attributes['data-arme-id'].value;
        let combatantId = event.currentTarget.attributes['data-combatant-id'].value;
        let arme = armesList[armeIndex];
        RdDUtility.rollInitiativeCompetence(combatantId, arme);
      });

    // combat
    await RdDTokenHud._configureSubMenu(html.find('.control-icon.target'), 'systems/foundryvtt-reve-de-dragon/templates/hud-actor-attaque.html', hudData,
      (event) => {
        let armeIndex = event.currentTarget.attributes['data-arme-id'].value;
        let arme = armesList[armeIndex];
        actor.rollArme(arme.data.competence, arme.name);
      });
  }

  /* -------------------------------------------- */
  static buildListeActionsCombat(combatant) {
    let armesList = RdDUtility.buildArmeList(combatant);
    for (let armeIndex = 0; armeIndex < armesList.length; armeIndex++) {
      armesList[armeIndex] = duplicate(armesList[armeIndex]);
      armesList[armeIndex].index = armeIndex;
    }
    return armesList;
  }

  /* -------------------------------------------- */
  static async _configureSubMenu(insertionPoint, template, hudData, onMenuItem) {
    const hud = $(await renderTemplate(template, hudData));
    const imgHud = hud.find('img.rdd-hud-togglebutton');
    const list = hud.find('div.rdd-hud-list');

    hud.toggleClass('active');
    HtmlUtility._showControlWhen(list, hud.hasClass('active'));

    imgHud.click(event => {
      hud.toggleClass('active');
      HtmlUtility._showControlWhen(list, hud.hasClass('active'));
    });

    list.find('.rdd-hud-menu').click(onMenuItem);

    insertionPoint.after(hud);
  }

}